local cjson = require "cjson"

ngx.req.read_body()
local args, err = ngx.req.get_post_args()
if not args then
    ngx.log(ngx.ERR, "failed to get post args")
    ngx.exit(ngx.HTTP_BAD_REQUEST)
end

if type(args) ~= "table" then
    ngx.log(ngx.ERR, "invalid args type")
    ngx.exit(ngx.HTTP_BAD_REQUEST)
end

local xdict       = {}
local username    = ""

for key, val in pairs(args) do
    if key == "username" then
        username = tostring(val)
    elseif key == "password" then
        xdict["password"] = tostring(val)
    end
end

if not username or not xdict["password"] then
    ngx.log(ngx.ERR, "missing username or password")
    ngx.exit(ngx.HTTP_BAD_REQUEST)
end

local res = ngx.location.capture("/thallium",
                { method = ngx.HTTP_POST,
                  body = cjson.encode(xdict),
                  vars = { username = string.gsub(username, "@", "-at-") }})

if res.status ~= ngx.HTTP_OK or res.body == nil then
    ngx.header["Content-Type"] = "text/html"
    ngx.say("invalid username or password")
    ngx.exit(ngx.HTTP_OK)
end

local body = cjson.decode(res.body)

if body['auth'] == nil or body['auth']['policies'] == nil then
    ngx.log(ngx.ERR, "missing thallium policies")
    ngx.exit(ngx.HTTP_INTERNAL_SERVER_ERROR)
end

local policies   = cjson.decode(ngx.var.THALLIUM_POLICIES)
local has_policy = false

if (type(policies) ~= 'table') then
    ngx.log(ngx.ERR, "invalid thallium policies")
    ngx.exit(ngx.HTTP_INTERNAL_SERVER_ERROR)
end

for index, value in ipairs(body['auth']['policies']) do
    if policies[value] ~= nil then
        has_policy = true
        break
    end
end

if has_policy == false then
    ngx.log(ngx.ERR, "no thallium policy found")
    ngx.exit(ngx.HTTP_INTERNAL_SERVER_ERROR)
end

args["password"] = ngx.var.FD_TOKEN

ngx.req.set_body_data(ngx.encode_args(args))
