function redis_connect()
    local redis = require "nginx.redis"
    local red = redis:new()

    red:set_timeout(ngx.var.redis_hk_timeout or 1000)

    local redis_host = ngx.var.redis_hk_host or '127.0.0.1'
    local redis_port = ngx.var.redis_hk_port or 6379

    local ok, err = red:connect(redis_host, redis_port)
    if not ok then
        ngx.log(ngx.ERR, "failed to connect to redis server: ", err)
        return false
    end

    return red
end

function redis_ping(red)
    local res, err = red:ping()
    if not res then
        if err then
            ngx.log(ngx.ERR, "redis error: " .. err)
        end
        return false
    end

    --ngx.log(ngx.ERR, "redis ping result: " .. res)

    return true
end

function redis_is_master(red)
    local res, err = red:info("replication")
    if not res then
        if err then
            ngx.log(ngx.ERR, "redis error: " .. err)
        end
        return false
    end

    --ngx.log(ngx.ERR, "redis replication result: " .. res)

    return res:find('role:master') ~= nil
end

function redis_health_check()
    local red = redis_connect()
    local is_master = ngx.var.redis_hk_is_master

    if is_master == nil then
        is_master = true
    end

    if not red then
        return false
    end

    if not redis_ping(red) then
        return false
    end

    if is_master == false then
        return true
    end

    if not redis_is_master(red) then
        return false
    end

    return true
end

if not redis_health_check() then
    ngx.exit(ngx.HTTP_INTERNAL_SERVER_ERROR)
end

ngx.exit(ngx.HTTP_OK)
